
void ADC_Init()
{
   // Direction of adc channels.
   DDRF = 0;

#ifdef INTERRUPT_MODE
   // Direction of interrupt.
   set_bit(DDRD, 7);
   clear_bit(PORTD, 7);
#endif

   // Initialize ADC.
   ADMUX = (1<<REFS0);
   ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
}

#ifdef BUFFERING_MODE
void ADC_EnableChan(char channel)
{
   char i;
   struct chanList *temp, *temp2;

   if(chCount == 0){
      // Make table element and start converting.
      channels = (channeltab + channel);
      channels->next = channels;
      channels->prev = channels;
      channels->channel = channel;
      ADC_NewConversion();
   }
   else{
      // Search through channels and check if the channel is already enabled.
      temp = channels;
      for(i=0; i<chCount; i++){
         // If channel already is enabled the function ends.
         if(temp->channel == channel) return;
         temp = temp->next;
      }

      // Make new channel.
      temp = (channeltab + channel);
      temp->channel = channel;

      // Inserts new channel in list.
      temp->next = channels->next;
      temp->prev = channels;
      channels->next = temp;
      channels->next->prev = temp;
   }

   // Increase number of channels;
   chCount++;
}

void ADC_DisableChan(char channel)
{
   char i, enabled;
   struct chanList *temp;

   temp = channels;
   enabled = 0;

   // Search through channels and check if the channel is enabled.
   for(i=0; i<chCount; i++){
      if(temp->channel == channel){
         enabled = 1;
         break;
      }
      temp = temp->next;
   }
   // If channel aren't enabled the function ends.
   if(enabled == 0) return;

   // Update pointers inside the list.
   temp->prev->next = temp->next;
   temp->next->prev = temp->prev;
   chCount--;

   if(chCount == 0) ADC_StopConversion();
}

void ADC_CheckBuffer()
{
   if(test_bit(ADCSRA, ADIF)){
      // Storing new data.
      dataL[channels->channel] = ADCL;
      dataH[channels->channel] = ADCH;

      // Start new conversion.
      ADC_NewConversion();
   }
}

void ADC_NewConversion()
{
   channels = channels->next;

   ADMUX = ADMUX & 0xe0;
   ADMUX = ADMUX | channels->channel;

   //Start conversion.
   set_bit(ADCSRA, ADSC);
}

void ADC_StopConversion()
{
   clear_bit(ADCSRA, ADSC);
}
#endif

#ifdef INTERRUPT_MODE
void ADC_StartConversion(char channel)
{
   ADMUX = ADMUX & 0xe0;
   ADMUX = ADMUX | channel;

   //Start conversion.
   set_bit(ADCSRA, ADSC);
}

signed char ADC_CheckConversion(unsigned char *data)
{
   if(test_bit(ADCSRA, ADIF)){
      // Clear status bit.
      set_bit(ADCSRA, ADIF);

      // Storing new data.
      data[1] = ADCL;
      data[0] = ADCH;

      // Finished.
      return 1;
   }
   // Not finished.
   return 0;
}
#endif
