#ifndef AM128_ADC
#define AM128_ADC

/*
Initialize the ADC part of the ATmega128.
*/
void ADC_Init();

#ifdef BUFFERING_MODE
// Data structures and functions for buffering mode.

// Struct for making a circular array for storing the channels that are enabled.
struct chanList{
	struct chanList *next;
	struct chanList *prev;
	char channel;
};

struct chanList channeltab[8]; // Static decleration of list elements.
struct chanList *channels; // Pointer to active element of the circular list.
char chCount; // Number of elements in the list.

// tables for storing the latest value for each channel.
unsigned char dataH[8], dataL[8]; 

// Enable one analog channel, if it's not already enabled. Enabled means that the ATmega128 will continuously convert analog values on this channel and store the digital values in a buffer.
void ADC_EnableChan(char channel);

// Disable one analog channel, if it's not already disabled.
void ADC_DisableChan(char channel);

// Check if the ongoing conversion is finished, if so store the data and start a new one.
void ADC_CheckBuffer();

// Starts conversion of the next channel in the array.
void ADC_NewConversion();

// Stops and ongoing conversions.
void ADC_StopConversion();

#endif

#ifdef INTERRUPT_MODE
// Interrupt mode.

// Starts a conversion of a given channel.
void ADC_StartConversion(char channel);

// Checks if a conversion is finished, if so the result are stored, and an interrupt is sent to the AVR32.
signed char ADC_CheckConversion(unsigned char *data);
#endif

#include "am128adc.c"

#endif //AM128_ADC
