#ifndef AM128_IO
#define AM128_IO

// Initialize the different parts of the I/O-card.
void IO_Init();

// Function that checks if an received data (rx) was a correct ack of the transmitted data (tx).
signed char IO_TestAck(unsigned char tx, unsigned char rx);

// Checks the received code and starts receiving a new command if code is valid.
signed char IO_NewCommand(unsigned char code);

/*
Analog in SPI message.

Number   Received from master    Returned to master
1        DUMMY                   code + 1
Wait for interrupt if in interrupt mode.
2        DUMMY                   vauleH
3        valueH + 1              vauleL
4        valueL + 1              DUMMY
5        MASTER_OK               SLAVE_OK
*/
signed char IO_AnalogIn(unsigned char code);

/*
Analog out SPI message.

Number   Received from master    Returned to master
1        valueH                  code + 1
2        valueL                  vauleH + 1
3        DUMMY                   vauleL + 1
4        MASTER_OK               SLAVE_OK
*/
signed char IO_AnalogOut(unsigned char code);

/*
Digital in SPI message.

Number   Received from master    Returned to master
1        DUMMY                   code + 1
2        DUMMY                   value
3        value + 1               DUMMY
4        MASTER_OK               SLAVE_OK
*/
signed char IO_DigitalIn(unsigned char code);

/*
Digital out SPI message.

Number   Received from master    Returned to master
1        mask                    code + 1
2        value                   mask + 1
3        DUMMY                   vaule + 1
4        MASTER_OK               SLAVE_OK
*/
signed char IO_DigitalOut(unsigned char code);

#ifdef BUFFERING_MODE
/*
Analog in enable SPI message. (Only used in buffering mode)

Number   Received from master    Returned to master
1        enable                  code + 1
2        DUMMY                   enable + 1
3        MASTER_OK               SLAVE_OK
*/
signed char IO_AnalogInEnable(unsigned char code);
#endif

/*
Analog out resolution SPI message.

Number   Received from master    Returned to master
1        resolution              code + 1
2        DUMMY                   resolution + 1
4        MASTER_OK               SLAVE_OK
*/
signed char IO_AnalogOutRes(unsigned char code);

// Includes.
#include "../spiByteCodes.h"
#include "am128spiSlave.h"
#include "am128pwm.h"
#include "am128adc.h"

#include "am128io.c"

#endif //AM128_IO
