void PWM_Init()
{
   // Set the PWM ports as output
   set_bit(DDRE, 3);
   set_bit(DDRE, 4);
   set_bit(DDRE, 5);
   set_bit(DDRB, 5);
   set_bit(DDRB, 6);
   set_bit(DDRB, 7);

   // Set the control registers for PWM to fast PWM, with no prescaler and clear output on compare match (non-inverting mode).
   TCCR1A =(1<<COM1A1) | (1<<COM1B1) | (1<<COM1C1) | (1<<WGM11);
   TCCR1B = (1<<WGM13) | (1<<WGM12) | (1<<CS10);

   TCCR3A =(1<<COM3A1) | (1<<COM3B1) | (1<<COM3C1) | (1<<WGM31);
   TCCR3B = (1<<WGM33) | (1<<WGM32) | (1<<CS30);

   // Set both PWM to have 8-bit resolution.
   PWM_SetResolution(0,8);
   PWM_SetResolution(1,8);

   // Set all PWM-outputs to zero.
   PWM_SetOutValue(0, 1, 0x00, 0x00);
   PWM_SetOutValue(0, 2, 0x00, 0x00);
   PWM_SetOutValue(0, 3, 0x00, 0x00);
   PWM_SetOutValue(1, 1, 0x00, 0x00);
   PWM_SetOutValue(1, 2, 0x00, 0x00);
   PWM_SetOutValue(1, 3, 0x00, 0x00);
}

void PWM_SetOutValue(char ch, char sch, char valueH, char valueL)
{
// Checks which PWM/timer that should be set. Either 0 or 1.
switch(ch){
   case 0:
      // Checks what channel of the PWM/timer. A=1, B=2 and C=3.
      switch(sch){
         case 0:
            OCR1AH = valueH;
            OCR1AL = valueL;
            break;
         case 1:
            OCR1BH = valueH;
            OCR1BL = valueL;
            break;
         case 2:
            OCR1CH = valueH;
            OCR1CL = valueL;
            break;
      }
      break;
   case 1:
      // Checks what channel of the PWM/timer. A=1, B=2 and C=3.
      switch(sch){
         case 0:
            OCR3AH = valueH;
            OCR3AL = valueL;
            break;
         case 1:
            OCR3BH = valueH;
            OCR3BL = valueL;
            break;
         case 2:
            OCR3CH = valueH;
            OCR3CL = valueL;
            break;
         }
      break;
   }
}

void PWM_SetResolution(char ch, char resolution)
{
   char resH, resL, i;
   resH = 0x00;
   resL = 0x00;

   // Finds values for the ICRn registers that gives the requested resolution.
   if(resolution <= 8){ // For resolution equal or less than 8-bits.
      for(i=0; i<resolution; i++){
         set_bit(resL, i);
      }
   }
   else{ // For resolution over 8-bits.
      resL = 0xff;
      for(i=0; i<resolution-8; i++){
         set_bit(resH, i);
      }
   }

   // Writes the values to the right ICRn register.
   switch(ch){
      case 0:
         ICR1H = resH;
         ICR1L = resL;
         break;
      case 1:
         ICR3H = resH;
         ICR3L = resL;
         break;
   }

   // Sets output on all subchannels to zero.
   PWM_SetOutValue(ch, 1, 0x00, 0x00);
   PWM_SetOutValue(ch, 2, 0x00, 0x00);
   PWM_SetOutValue(ch, 3, 0x00, 0x00);
}
