void UART_Init()
{
#ifdef RS232_DEBUG
   // Set direction registers.
   set_bit(DDRD, 3);
   clear_bit(DDRD, 2);

   // BAUD rate = 19.2k.
   UBRR1H = 0;
   UBRR1L = 51;

   // Enable both transmit and receive.
   UCSR1B = (1<<RXEN1) | (1<<TXEN1);

   // Use 8 bit data, no parity and 2 stop bits.
   UCSR1C = (1<<UCSZ11) | (1<<UCSZ10) | (1<<USBS1);
#endif
}

void UART_TxByte(char data)
{
#ifdef RS232_DEBUG
   // Wait until last message is sent, then send this one.
   loop_bit_is_set(UCSR1A, UDRE1);
   UDR1=data;
#endif
}

void UART_TxSpace()
{
#ifdef RS232_DEBUG
   UART_TxByte(32);
#endif
}

void UART_TxNewLine()
{
#ifdef RS232_DEBUG
   UART_TxByte('\n');
   UART_TxByte('\r');
#endif
}


void UART_TxDecimal(unsigned char value)
{
#ifdef RS232_DEBUG
   char tx[3], i;

   //tx[0]
   tx[0] = '0';
   for(i=0;i<3;i++){
      if(value > 99){
         tx[0] = tx[0] + 1;
         value = value - 100;
      }
   }

   //tx[1]
   tx[1] = '0';
   for(i=0;i<10;i++){
      if(value > 9){
         tx[1] = tx[1] + 1;
         value = value - 10;
      }
   }

   //tx[2]
   tx[2] = '0';
   for(i=0;i<10;i++){
      if(value > i){
         tx[2] = tx[2] + 1;
      }
   }

   //Print
   for(i=0;i<3;i++){
      UART_TxByte(tx[i]);
   }
#endif
}

void UART_Print(char symbol, unsigned char value)
{
#ifdef RS232_DEBUG
   UART_TxByte(symbol);
   UART_TxByte(':');
   UART_TxSpace();
   UART_TxDecimal(value);
   UART_TxNewLine();
#endif
}


