
long avr32io_GetAnalogIn(char channel)
{
   int fd, ret;
   char devFile[13];
   unsigned char data[2];
   unsigned short result;

   pthread_mutex_lock(&spiMutex);

   // Open the correct device node.
   sprintf(devFile, "/dev/avr32Ain%i", channel);
   fd = open(devFile, O_RDWR);
   if(fd < 0){
      DEBUG("avr32io_user: error on open: %i\n", fd);
      return fd;
   }

   // Read data from device node and close it.
   ret = read(fd, data, 2);
   close(fd);
   if(ret < 0){
      DEBUG("avr32io_user: error on read: %i\n", ret);
      return ret;
   }

   pthread_mutex_unlock(&spiMutex);

   // Convert result and return.
   result = (data[0] << 8) + data[1];
   return (long)(result);
}

short avr32io_SetAnalogOut(char channel, char subchannel, unsigned short value)
{
   int ret, fd;
   char devFile[14];
   unsigned char data[3];

   // Prepare data to send.
   data[0] = subchannel;
   data[1] = value >> 8;
   data[2] = value;

   pthread_mutex_lock(&spiMutex);

   // Open the correct device node.
   sprintf(devFile, "/dev/avr32Aout%i", channel);
   fd = open(devFile, O_RDWR);
   if(fd < 0){
      DEBUG("avr32io_user: error on open: %i\n", fd);
      return fd;
   }

   // Write data to device node and close it.
   ret = write(fd, data, 3);
   close(fd);
   if(ret < 0){
      DEBUG("avr32io_user: error on write: %i\n", ret);
      return ret;
   }

   pthread_mutex_unlock(&spiMutex);

   return 0;
}

short avr32io_SetAnalogOutDouble(char channel, char subchannel, double value)
{
   // Converting the double value to integer and call the analog out function.
   return avr32io_SetAnalogOut(channel, subchannel, (int)(aoutMultiplier[channel] * value));
}

short avr32io_GetDigitalIn()
{
   int fd, ret;
   unsigned char data;

   pthread_mutex_lock(&spiMutex);

   // Open the correct device node.
   fd = open("/dev/avr32Din", O_RDWR);
   if(fd < 0){
      DEBUG("avr32io_user: error on open: %i\n", fd);
      return fd;
   }

   // Read data from device node and close it.
   ret = read(fd, &data, 1);
   close(fd);
   if(ret < 0){
      DEBUG("avr32io_user: error on read: %i\n", ret);
      return ret;
   }

   pthread_mutex_unlock(&spiMutex);

   // Return result.
   return (short)(data);
}

short avr32io_GetDigitalInBit(char bit)
{
   short data;
   unsigned char ucData;

   // Get digital in data.
   data = avr32io_GetDigitalIn();
   if(data < 0){
      return data;
   }

   // Check the bit, and return the result.
   ucData = (unsigned char)(data);
   if((ucData & (1 << bit)) > 0){
      return 1;
   }
   else{
      return 0;
   }
}

short avr32io_SetDigitalOut(unsigned char mask, unsigned char value)
{
   int fd, ret;
   unsigned char data[2];

   //Prepare data to send.
   data[0] = mask;
   data[1] = value;

   pthread_mutex_lock(&spiMutex);

   // Open the correct device node.
   fd = open("/dev/avr32Dout", O_RDWR);
   if(fd < 0){
      DEBUG("avr32io_user: error on open: %i\n", fd);
      return fd;
   }

   // Write data to the device node and close it.
   ret = write(fd, data, 2);
   close(fd);
   if(ret < 0){
      DEBUG("avr32io_user: error on write: %i\n", ret);
      return ret;
   }

   pthread_mutex_unlock(&spiMutex);

   return 0;
}

short avr32io_SetDigitalOutBit(char bit, char value)
{
   unsigned char mask, newValue;

   // Prepare data for new function call.
   mask = 1 << bit;
   newValue = 0;
   if (value > 0){
      newValue = 1 << bit;
   }

   // Write data and return.
   return avr32io_SetDigitalOut(mask, newValue);
}

short avr32io_SetAnalogOutResolution(char channel, char resolution)
{
   int fd, ret;
   char devFile[14];

   aoutMultiplier[channel] = (double)((pow(2, resolution) - 1) / 5);

   pthread_mutex_lock(&spiMutex);

   // Open the correct device node.
   sprintf(devFile, "/dev/avr32Aout%i", channel);
   fd = open(devFile, O_RDWR);
   if(fd < 0){
      DEBUG("avr32io_user: error on open: %i\n", fd);
      return fd;
   }

   // Perform IOCTL command on the device node and close it.
   ret = ioctl(fd, AVR32IO_IOCTL_RES_AOUT, resolution);
   close(fd);
   if(ret < 0){
      DEBUG("avr32io_user: error on write: %i\n", ret);
      return ret;
   }

   pthread_mutex_unlock(&spiMutex);

   return 0;
}

short avr32io_EnableAnalogIn(char channel, char enable)
{
   int fd, ret;
   char devFile[14];

   pthread_mutex_lock(&spiMutex);

   // Open the correct device node.
   sprintf(devFile, "/dev/avr32Ain%i", channel);
   fd = open(devFile, O_RDWR);
   if(fd < 0){
      DEBUG("avr32io_user: error on open: %i\n", fd);
      return fd;
   }

   // Perform IOCTL command on the device node and close it.
   ret = ioctl(fd, AVR32IO_IOCTL_EN_AIN, enable);
   close(fd);
   if(ret < 0){
      DEBUG("avr32io_user: error on write: %i\n", ret);
      return ret;
   }

   pthread_mutex_unlock(&spiMutex);

   return 0;
}

