#ifndef __AVRDRIVER_H
#define __AVRDRIVER_H

#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <math.h>
#include <pthread.h>
#include <linux/ioctl.h>

//Defines IOCTL numbers.
#define AVR32IO_IOCTL_MAGIC 'k'
#define AVR32IO_IOCTL_RES_AOUT _IOW(AVR32IO_IOCTL_MAGIC, 0, int)
#define AVR32IO_IOCTL_EN_AIN _IOW(AVR32IO_IOCTL_MAGIC, 1, int)

// Defines if driver should print out debugging information.
//#define VERBOSE_DEBUG
#ifdef VERBOSE_DEBUG
	#define DEBUG(fmt, args...) { printf(fmt, ## args); }
#else
	#define DEBUG(fmt, args...)
#endif

// Get integer result from analog in channel.
long avr32io_GetAnalogIn(char channel);

// Set integer value to analog out channel and subchannel.
short avr32io_SetAnalogOut(char channel, char subchannel, unsigned short value);

// Set double value to analog out channel and subchannel.
short avr32io_SetAnalogOutDouble(char channel, char subchannel, double value);

// Get digital in byte.
short avr32io_GetDigitalIn();

// Get single bit of digital in.
short avr32io_GetDigitalInBit(char bit);

// Set digital out with a mask and value.
short avr32io_SetDigitalOut(unsigned char mask, unsigned char value);

// Set just a single digital out bit.
short avr32io_SetDigitalOutBit(char bit, char value);

// Set a new analog out resolution for a channel.
short avr32io_SetAnalogOutResolution(char channel, char resolution);

// Enables or disables an analog in channel (will only work in BUFFERING_MODE)
short avr32io_EnableAnalogIn(char channel, char enable);

// Containing a double value for as fast as possible convert 0-5 double value to integer with the correct resolution.
double aoutMultiplier[2];

// Mutex used so only one thread can access the SPI at the same time.
pthread_mutex_t spiMutex = PTHREAD_MUTEX_INITIALIZER;

#include "avr32io_driver.c"

#endif // __AVRDRIVER_H
