#include <stdio.h>
#include <pthread.h>
#include <math.h>
void Init_IO(){

   if(!IO_Started){

   // Initialize periodic tasks.
   InitPeriodicTasks();

   // Starts analog output worker thread.
   aoutCmdListFirst = 0;
   aoutCmdListLast = 0;
   workCount = 0;
   pthread_create(&aoutWorker, NULL, Worker_Aout, NULL);

   // Marks that this function have been run.
   IO_Started = 1;
   }
}

int avr32io_InitAnalogIn(int chanNumber, double period)
{
   // Enables selected channel.
	struct sAinChan *chan;
	chan = ainChanList + chanNumber;
	avr32io_EnableAnalogIn(chanNumber, 1);

   // Start periodic thread for sampling the analog input channel.
	pthread_mutex_init(&(chan->mutex), NULL);
	StartPeriodicThread(&(chan->samplingTask), (int)(period/MASTER_TIMER_PERIOD), Sampling_Ain, chan);
}

void *Sampling_Ain(void *ptr)
{
	int count = 0;
	struct sAinChan *chan;
	chan = ptr;
	chan->value = 0;

   // Forever loop for sampling analog input.
	while(1){
      // Wait for periode.
		WaitPeriodicTask(&(chan->samplingTask));

      // Lock mutex, update value and unlock mutex.
		pthread_mutex_lock(&(chan->mutex));
		chan->value = (float)(avr32io_GetAnalogIn(chan->chanNumber) * 0.004888);
		pthread_mutex_unlock(&(chan->mutex));
	}
}

int avr32io_GetAnalogInThread(int chanNumber, double *data)
{
	double result;
	struct sAinChan *chan;
	chan = ainChanList + chanNumber;

   // Lock mutex, save value and unlock mutex.
	pthread_mutex_lock(&(chan->mutex));
	result = chan->value;
	pthread_mutex_unlock(&(chan->mutex));

	// For some reason returning a double didn't work with RTW
	*data = result;
	return 0;
}

int avr32io_SetAnalogOutThread(int chanNumber, int subchanNumber, double value)
{
   // Makes new list item.
   struct sAoutCmd *temp;
   temp = malloc(sizeof(struct sAoutCmd));
   temp->chanNumber = chanNumber;
   temp->subchanNumber = subchanNumber;
   temp->value = value;
   temp->next = NULL;

   // Lock mutex, add item to the list and unlock mutex.
   pthread_mutex_lock(&cmdListMutex);
   if(aoutCmdListFirst == NULL){
      // List empty.
      aoutCmdListFirst = temp;
   }
   else{
      // List not empty.
      aoutCmdListLast->next = temp;
   }
   aoutCmdListLast = temp;
   workCount++;
   pthread_mutex_unlock(&cmdListMutex);

   // Signaling the worker thread.
   pthread_cond_signal(&workCond);
	return 0;
}


void *Worker_Aout()
{
   struct sAoutCmd *temp;

   // Forever loop.
	while(1){
		// Wait for work.
		pthread_mutex_lock(&workMutex);
		pthread_cond_wait(&workCond, &workMutex);
		pthread_mutex_unlock(&workMutex);

      // Executes commands until list is empty.
		while(workCount>0){
         // Locks mutex, grab first list item and unlock mutex.
			pthread_mutex_lock(&cmdListMutex);
			temp = aoutCmdListFirst;
			aoutCmdListFirst = temp->next;
			workCount--;
			pthread_mutex_unlock(&cmdListMutex);

         // Send value to I/O-card and free the list items memory.
			avr32io_SetAnalogOut(temp->chanNumber, temp->subchanNumber, (int)(aoutMultiplier[temp->chanNumber] * temp->value));
			free(temp);
		}
	}
}


