#include "periodicTask.h"
#include "avr32io_driver.h"

#define MASTER_TIMER_PERIOD 0.001

// Structure containing information about each of the analog input channels.
struct sAinChan{
	int chanNumber;
	double value;
	pthread_mutex_t mutex;
	struct sPeriodicTask samplingTask;
};
struct sAinChan ainChanList[8];

// Structure for list of command the worker thread has to execute.
struct sAoutCmd{
   int chanNumber;
	int subchanNumber;
	double value;
   struct sAoutCmd *next;
};
struct sAoutCmd *aoutCmdListFirst, *aoutCmdListLast;

pthread_t aoutWorker; // Worker thread for analog out.
pthread_mutex_t cmdListMutex = PTHREAD_MUTEX_INITIALIZER; // Mutex for command list.
pthread_mutex_t workMutex = PTHREAD_MUTEX_INITIALIZER; // Mutex for new command signal.
pthread_cond_t workCond = PTHREAD_COND_INITIALIZER; // New command signal.
int workCount = 0; // Number of commands in list.
int IO_Started = 0; // Protects the Init_IO() function from beeing run several times.

// Initialize IO.
void Init_IO();

// Initialize an analog input channel, and start a periodic thread that samples this channel.
int avr32io_InitAnalogIn(int chanNumber, double period);

// Function for analog input sampling thread.
void *Sampling_Ain(void *ptr);

// Function for retriving analog input value from buffer. For some reason returning a double didn't work with the RTW program, so the value will be returned in a pointer instead.
int avr32io_GetAnalogInThread(int chanNumber, double *data);

// Give command to the worker thread about a new analog output value.
int avr32io_SetAnalogOutThread(int chanNumber, int subchanNumber, double value);

// Function for the worker thread, used to execute commands with new analog output values.
void *Worker_Aout();

#include "avr32io_threads.c"
