int InitPeriodicTasks(){
   static int initialized;

   if(initialized > 0) return -1;
   // Initialize the timer.
   signal(SIGALRM,fTimer);
   rttimer.it_value.tv_sec = 0;
   rttimer.it_value.tv_usec = 1;
   rttimer.it_interval.tv_sec = 0;
   rttimer.it_interval.tv_usec = 1000;
   setitimer(ITIMER_REAL, &rttimer, NULL);

   // Initialize a empty taks table.
   rootTask = NULL;
   initialized = 1;

   // Return zero.
   return 0;
}

void fTimer(){
   // Variables
   static long count;
   struct sPeriodicTask *task;

   // Increase the counter.
   count++;

   // Walks through the tasks and signal the ones that need to run.
   task = rootTask;

   while(task != NULL){
      if((count % (task->periode)) == 0){
         // Signal a task.
         pthread_cond_signal(&(task->cond));
      }
      // Next task.
      task = task->next;
   }
}

int StartPeriodicTask(struct sPeriodicTask *pTask, int periode)
{
   // Variables
   struct sPeriodicTask *newTask, *previousTask;

   // The new task will be pointing to nothing.
   pTask->next = NULL;
   if(rootTask == NULL){
      // If rootTask doesn't point at anything, this task is the first task.
      rootTask = pTask;
   }
   else{
      // If rootTask point at something, the first task that doesnt point at a next task has to be found.
      previousTask = rootTask;
      newTask = rootTask->next;
      while(newTask != NULL){
         // Will quit when a empty next pointer is found.
         previousTask = newTask;
         newTask = newTask->next;
      }

      // The empty newTask will now point to pTask, and the former newest task will point to the new.
      newTask = pTask;
      previousTask->next = newTask;
   }
   // Initialize the content of the new task.
   pTask->periode = periode;
   pthread_cond_init(&(pTask->cond), NULL);
   pthread_mutex_init(&(pTask->mutex), NULL);
   return 0;
}

int StartPeriodicThread(struct sPeriodicTask *pTask, int periode, void *(*__start_routine) (void *), void* data)
{
   int ret;

   // Start a periodic task and a thread.
   ret = StartPeriodicTask(pTask, periode);
   pthread_create(&(pTask->thread), NULL, (*__start_routine), data);
   return ret;
}


int WaitPeriodicTask(struct sPeriodicTask *pTask)
{
   // Wait for the signal to the given task.
   pthread_mutex_lock(&(pTask->mutex));
   pthread_cond_wait(&(pTask->cond), &(pTask->mutex));
   pthread_mutex_unlock(&(pTask->mutex));
   return 0;
}
