#ifndef DEF_PTASK
#define DEF_PTASK

// includes
#include <signal.h>
#include <pthread.h>
#include <sys/time.h>

// Struct that represents one periodic task.
struct sPeriodicTask{
	struct sPeriodicTask *next; // Pointer to the next periodc task.
	int periode; // The periode of the task.
	pthread_t thread; // The thread thats running the task.
	pthread_cond_t cond; // The condition that controls the task.
	pthread_mutex_t mutex; // The mutex protecting the condition.
};

// Struct for the timer.
struct itimerval rttimer;

// Struct for the root task.
struct sPeriodicTask *rootTask;

// This function initialize the periodic task system.
int InitPeriodicTasks();

//Function that runs every millisecond, signaling all the periodic tasks.
void fTimer();

// This function starts a new periodic task. Started tasks can't be stopped.
int StartPeriodicTask(struct sPeriodicTask *pTask, int periode);

// This function starts a new periodic thread. Started threads can't be stopped.
int StartPeriodicThread(struct sPeriodicTask *pTask, int periode, void *(*function) (void *), void* data);

// Used inside the periodic task/thread to sleep until next periode.
int WaitPeriodicTask(struct sPeriodicTask *pTask);

#include "periodicTask.c"

#endif
