
int Cmd_WaitForAM128(u8 code)
{
   int count = 0;
   u8 ready;

   // Loops until it receives the correct code from ATMega128.
   while(ready != SLAVE_READY){
      SPI_MasterTransfer(&ready, &code, 1);
      if(count++ > 100) return -EIO;
   }
   return count;
}

int Cmd_GetAIn(u8 *toUser, int ch)
{
   int ret;

   // check for valid channel.
   if(ch < 0) return -EIO;

   // Synchronize with ATMega128
   ret = Cmd_WaitForAM128(CH_TO_CODE_AIN(ch));
   DEBUG("code: %i count: %i\n", CH_TO_CODE_AIN(ch), ret);
   if (ret < 0) return ret;

   // Send DUMMY, receive code ack.
   ret = SPI_MasterTransferByte(DUMMY);
   DEBUG("dummy: %i code+1: %i\n", DUMMY, ret);
   if(ret != CH_TO_CODE_AIN(ch) + 1)  return -EIO;

#ifdef INTERRUPT_MODE
   // Wait for interrupt.
   device->irqFlag = 0;
   DEBUG("wait\n");
   ret = wait_event_interruptible_timeout(wq, device->irqFlag != 0, 10000);
   if(ret < 0) return -ERESTARTSYS;
   DEBUG("interrupt\n");
#endif

   // Send DUMMY, receive data0.
   ret = SPI_MasterTransferByte(DUMMY);
   DEBUG("dummy: %i data0: %i\n", DUMMY, ret);
   if(ret < 0)  return ret;
   toUser[0] = (u8)ret;

   // Send data0 ack, receive data1.
   ret = SPI_MasterTransferByte((u8)(toUser[0] + 1));
   DEBUG("data0+1: %i data1: %i\n", ((u8)(toUser[0]) + 1), ret);
   if(ret < 0)  return ret;
   toUser[1] = (u8)ret;

   // Send data1 ack, receive DUMMY.
   ret = SPI_MasterTransferByte((u8)(toUser[1] + 1));
   DEBUG("data0+1: %i dummy: %i\n", ((u8)(toUser[1]) + 1), ret);
   if(ret != DUMMY)  return -EIO;

   // Confirm command.
   ret = SPI_MasterTransferByte(MASTER_OK);
   DEBUG("masterOK: %i slaveOK: %i\n", MASTER_OK, ret);
   if(ret != SLAVE_OK) return -EIO;

   return 0;
}

int Cmd_SetAOut(u8 *fromUser, int ch, int sch)
{
   int ret;
   unsigned char tx[3], expectedRx[3];

   // check for valid channel.
   if(ch < 0) return -EIO;

   // Synchronize with ATMega128
   ret = Cmd_WaitForAM128(CH_TO_CODE_AOUT(ch, sch));
   DEBUG("code: %i count: %i\n", CH_TO_CODE_AOUT(ch, sch), ret);
   if (ret < 0) return ret;

   // Prepare to send several bytes.
   tx[0] = fromUser[0];
   tx[1] = fromUser[1];
   tx[2] = DUMMY;
   expectedRx[0] = CH_TO_CODE_AOUT(ch, sch) + 1;
   expectedRx[1] = fromUser[0] + 1;
   expectedRx[2] = fromUser[1] + 1;

   // Send data and check returns.
   ret = SPI_SendTableOfBytes(tx, expectedRx, 3);
   if(ret < 0) return ret;

   // Confirm command.
   ret = SPI_MasterTransferByte(MASTER_OK);
   DEBUG("masterOK: %i slaveOK: %i\n", MASTER_OK, ret);
   if(ret != SLAVE_OK) return -EIO;

   return 0;
}

int Cmd_GetDIn(u8 *toUser)
{
   int ret;

   // Synchronize with ATMega128
   ret = Cmd_WaitForAM128(CODE_DIN);
   DEBUG("1: code: %i count: %i\n", CODE_DIN, ret);
   if (ret < 0) return ret;

   // Send DUMMY, receive code ack.
   ret = SPI_MasterTransferByte(DUMMY);
   DEBUG("2: dummy: %i code+1: %i\n", DUMMY, ret);
   if(ret != CODE_DIN + 1)  return -EIO;

   // Send DUMMY, receive data0.
   ret = SPI_MasterTransferByte(DUMMY);
   DEBUG("3: dummy: %i data0: %i\n", DUMMY, ret);
   if(ret < 0)  return ret;
   toUser[0] = (u8)ret;

   // Send data0 ack, receive DUMMY.
   ret = SPI_MasterTransferByte(toUser[0] + 1);
   DEBUG("4: data0+1: %i dummy: %i\n", ((u8)(toUser[0]) + 1), ret);
   if(ret != DUMMY)  return -EIO;

   // Confirm command.
   ret = SPI_MasterTransferByte(MASTER_OK);
   DEBUG("masterOK: %i slaveOK: %i\n", MASTER_OK, ret);
   if(ret != SLAVE_OK) return -EIO;

   return 0;
}

int Cmd_SetDOut(u8 *fromUser)
{
   int ret;
   unsigned char tx[3], expectedRx[3];

   // Synchronize with ATMega128
   ret = Cmd_WaitForAM128(CODE_DOUT);
   DEBUG("code: %i count: %i\n", CODE_DOUT, ret);
   if (ret < 0) return ret;

   // Prepare to send several bytes.
   tx[0] = fromUser[0];
   tx[1] = fromUser[1];
   tx[2] = DUMMY;
   expectedRx[0] = CODE_DOUT + 1;
   expectedRx[1] = fromUser[0] + 1;
   expectedRx[2] = fromUser[1] + 1;

   // Send data and check returns.
   ret = SPI_SendTableOfBytes(tx, expectedRx, 3);
   if(ret < 0) return ret;

   // Confirm command.
   ret = SPI_MasterTransferByte(MASTER_OK);
   DEBUG("masterOK: %i slaveOK: %i\n", MASTER_OK, ret);
   if(ret != SLAVE_OK) return -EIO;

   return 0;
}

#ifdef BUFFERING_MODE
int Cmd_EnableAIn(u8 enable, int ch)
{
   int ret;
   unsigned char tx[2], expectedRx[2];

   // check for valid channel.
   if(ch < 0) return -EIO;

   // Synchronize with ATMega128
   ret = Cmd_WaitForAM128(CH_TO_CODE_AIN_EN(ch));
   DEBUG("code: %i count: %i\n", CH_TO_CODE_AIN_EN(ch), ret);
   if (ret < 0) return ret;

   // Prepare to send several bytes.
   tx[0] = enable;
   tx[1] = DUMMY;
   expectedRx[0] = CH_TO_CODE_AIN_EN(ch) + 1;
   expectedRx[1] = enable + 1;

   // Send data and check returns.
   ret = SPI_SendTableOfBytes(tx, expectedRx, 2);
   if(ret < 0) return ret;

   // Confirm command.
   ret = SPI_MasterTransferByte(MASTER_OK);
   DEBUG("masterOK: %i slaveOK: %i\n", MASTER_OK, ret);
   if(ret != SLAVE_OK) return -EIO;

   return 0;
}
#endif

int Cmd_SetAOutResolution(u8 resol, int ch)
{
   int ret;
   unsigned char tx[2], expectedRx[2];

   // Synchronize with ATMega128
   ret = Cmd_WaitForAM128(CH_TO_CODE_AOUT_RES(ch));
   DEBUG("code: %i count: %i\n", CH_TO_CODE_AOUT_RES(ch), ret);
   if (ret < 0) return ret;

   // Prepare to send several bytes.
   tx[0] = resol;
   tx[1] = DUMMY;
   expectedRx[0] = CH_TO_CODE_AOUT_RES(ch) + 1;
   expectedRx[1] = resol + 1;

   // Send data and check returns.
   ret = SPI_SendTableOfBytes(tx, expectedRx, 2);
   if(ret < 0) return ret;

   // Confirm command.
   ret = SPI_MasterTransferByte(MASTER_OK);
   DEBUG("masterOK: %i slaveOK: %i\n", MASTER_OK, ret);
   if(ret != SLAVE_OK) return -EIO;

   return 0;
}

#ifdef INTERRUPT_MODE
irqreturn_t irqFunk(int irq, void *dev_id, struct pt_regs *regs)
{
   // Wake up queue waiting for interrupt.
   DEBUG("received interrupt\n");
   device->irqFlag = 1;
   wake_up_interruptible(&wq);
   return IRQ_HANDLED;
}
#endif
