#ifndef __KM_AVR32IO_CMD_H
#define __KM_AVR32IO_CMD_H

/*
Sends code until ready code is received.
*/
int Cmd_WaitForAM128(u8 code);

/*
Sends SPI message for Analog in

Number   Send to slave     Returned by slave
1        code              READY
2        DUMMY             code +1
Wait for interrupt in interrupt mode.
3        DUMMY             vauleH
4        valueH + 1        vauleL
5        valueL + 1        DUMMY
6        MASTER_OK         SLAVE_OK
*/
int Cmd_GetAIn(u8 *toUser, int ch);

/*
Sends SPI message for Analog out.

Number   Received from master    Returned to master
1        code                    READY
2        valueH                  code + 1
3        valueL                  vauleH + 1
4        DUMMY                   vauleL + 1
5        MASTER_OK               SLAVE_OK
*/
int Cmd_SetAOut(u8 *fromUser, int ch, int sch);

/*
Sends SPI message for Digital in

Number   Send to slave     Returned by slave
1        code              READY
2        DUMMY             code +1
3        DUMMY             vaule
4        value + 1         DUMMY
5        MASTER_OK         SLAVE_OK
*/
int Cmd_GetDIn(u8 *toUser);

/*
Sends SPI message for Digital out.

Number   Received from master    Returned to master
1        code                    READY
2        mask                    code + 1
3        value                   mask + 1
4        DUMMY                   vaule + 1
5        MASTER_OK               SLAVE_OK
*/
int Cmd_SetDOut(u8 *fromUser);

#ifdef BUFFERING_MODE
/*
Sends SPI message for enable analog in.

Number   Received from master    Returned to master
1        code                    READY
2        enable                  code + 1
3        DUMMY                   enable + 1
5        MASTER_OK               SLAVE_OK
*/
int Cmd_EnableAIn(u8 en, int ch);
#endif

/*
Sends SPI message for analog out resolution.

Number   Received from master    Returned to master
1        code                    READY
2        resolution              code + 1
3        DUMMY                   resolution + 1
5        MASTER_OK               SLAVE_OK
*/
int Cmd_SetAOutResolution(u8 resol, int ch);

#ifdef INTERRUPT_MODE
DECLARE_WAIT_QUEUE_HEAD(wq); // Makes a wait queue.

// Function that runs on interrupt.
irqreturn_t irqFunk(int irq, void *dev_id, struct pt_regs *regs);
#endif

#include "avr32io_Cmd.c"

#endif // __KM_AVR32IO_CMD_H
