#ifndef SPI_BYTES_CODES
#define SPI_BYTES_CODES

/************************************************************************
 * The code bytes are generated the following rules.
 *
 * - No code can have the value 0, because thats used as dummy byte.
 * - Bit 7 (MSB):
 *   - Equals 0 for commands that sends or receives data.
 *   - Equals 1 for control commands.
 * - Bit 6:
 *   - Equals 0 for commands for digital IO.
 *   - Equals 1 for commands for analog IO.
 * - Bit 5:
 *   - Equals 0 for commands for output.
 *   - Equals 1 for commands for input.
 * - Bit 4 and 3: 
 *   - Equals normally 11.
 *   - Can have other values if bytes 5-7 is the same for multiple commands.
 * - Bit 2 to 0 (LSB):
 *   - For analog out, bit 2 equals channel and bit 0-1 equals subchannel.
 *   - For analog in, bit 0-2 equals channel.
 *   - For digital IO, bit 0-2 equals 000.
 ************************************************************************/

// codes
#define SLAVE_READY              0xaa /* 10101010 */
#define SLAVE_OK                 0x11 /* 00010001 */
#define MASTER_OK                0x22 /* 00100010 */
#define DUMMY                    0x00 /* 00000000 */

// Data command codes.
#define CODE_AIN                 0x78 /* 01111000 + ch */
#define CODE_AIN_REQ             0x68 /* 01101000 + ch */
#define CODE_AIN_RET             0x70 /* 01110000 + ch */
#define CODE_AOUT                0x58 /* 01011000 + ch<<2 + sch */
#define CODE_DIN                 0x38 /* 00111000 */
#define CODE_DOUT                0x18 /* 00011000 */

// Control command codes.
#define CODE_AIN_EN              0xf8 /* 11111000 + ch */
#define CODE_AOUT_RES            0xd8 /* 11011000 + ch>>2 */

// Macros for finding channels from codes.
#define CODE_TO_CH_AIN(code)     (code & 0x7)
#define CODE_TO_CH_AOUT(code)    ((code & 0x4) >> 2)
#define CODE_TO_SCH_AOUT(code)   (code & 0x3)

// Macros for finding codes from channels.
#define CH_TO_CODE_AIN(ch)       (CODE_AIN + ch)
#define CH_TO_CODE_AIN_EN(ch)    (CODE_AIN_EN + ch)
#define CH_TO_CODE_AIN_REQ(ch)   (CODE_AIN_REQ + ch)
#define CH_TO_CODE_AIN_RET(ch)   (CODE_AIN_RET + ch)
#define CH_TO_CODE_AOUT(ch, sch) (CODE_AOUT + (ch << 2) + sch)
#define CH_TO_CODE_AOUT_RES(ch)  (CODE_AOUT_RES + (ch << 2))

#endif //SPI_BYTES_CODES
